"""Console script for network_connectivity_tester."""
import sys
import click

from network_connectivity_tester.awsclient import ConnectivityTestClient


@click.command()
@click.option("--daemonize", is_flag=True, help="Daemonize worker process.")
@click.option("--region", default="us-east-2", help="AWS region.")
@click.option(
    "--registration-queue-name",
    required=True,
    help="Registration queue name that the orchestrator process uses.",
)
def main(daemonize, region, registration_queue_name):
    """Console script for network_connectivity_tester."""

    ct = ConnectivityTestClient(
        region=region, registration_queue_name=registration_queue_name
    )
    if daemonize:
        with daemon.DaemonContext():
            ct.loop()
    else:
        ct.loop()

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import json
import logging
import subprocess

import boto3
import botocore
import urllib.request

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


class ConnectivityTestClient(object):
    def __init__(self, region=None, registration_queue_name=None):
        self.instance_id = (
            urllib.request.urlopen(
                "http://169.254.169.254/latest/meta-data/instance-id"
            )
            .read()
            .decode()
        )
        nic_macs = [
            mac.decode().strip("/")
            for mac in urllib.request.urlopen(
                "http://169.254.169.254/latest/meta-data/network/interfaces/macs"
            ).readlines()
        ]
        self.nic_config = {}

        # list of ports socat is running on
        self._listen = {}

        for mac in nic_macs:
            self.nic_config["vpc-id"] = (
                urllib.request.urlopen(
                    "http://169.254.169.254/latest/meta-data/network/interfaces/macs/%s/vpc-id"
                    % mac
                )
                .read()
                .decode(),
            )
            self.nic_config["security-groups"] = (
                [
                    el.decode()
                    for el in urllib.request.urlopen(
                        "http://169.254.169.254/latest/meta-data/network/interfaces/macs/%s/security-groups"
                        % mac
                    ).readlines()
                ],
            )

            try:
                self.nic_config["ipv4-associations"] = [
                    el.decode()
                    for el in urllib.request.urlopen(
                        "http://169.254.169.254/latest/meta-data/network/interfaces/macs/%s/ipv4-associations"
                        % mac
                    ).readlines()
                ]
            except urllib.error.HTTPError:
                pass

            try:
                self.nic_config["local-ipv4s"] = [
                    el.decode()
                    for el in urllib.request.urlopen(
                        "http://169.254.169.254/latest/meta-data/network/interfaces/macs/%s/local-ipv4s"
                        % mac
                    ).readlines()
                ]
            except urllib.error.HTTPError:
                pass

        self.command_queue_name = "%s-%s" % (registration_queue_name, self.instance_id)
        self.registration_queue_name = registration_queue_name
        self.sqs = boto3.resource("sqs", region_name=region)

        self.registration_queue = None
        self.command_queue = None
        self.register()

    def register(self):
        try:
            # Get the queue. This returns an SQS.Queue instance
            self.registration_queue = self.sqs.get_queue_by_name(
                QueueName=self.registration_queue_name
            )

            # You can now access identifiers and attributes
            assert self.registration_queue.url
        except botocore.exceptions.ClientError as err:
            self.registration_queue = self.sqs.create_queue(
                QueueName=self.registration_queue_name
            )

        self.command_queue = self.sqs.create_queue(QueueName=self.command_queue_name)

        registration_payload = {
            "instance_id": self.instance_id,
            "command_queue_name": self.command_queue_name,
            "command_queue_url": self.command_queue.url,
            "nic_config": self.nic_config,
        }

        self.registration_queue.send_message(
            MessageBody=json.dumps(registration_payload),
            MessageAttributes={
                "InstanceRegistration": {
                    "StringValue": self.instance_id,
                    "DataType": "String",
                }
            },
        )

    def response(self, success, body, message_id):
        logger.info("Sending response %s to message %s" % (success, message_id))
        self.registration_queue.send_message(
            MessageBody=body,
            MessageAttributes={
                "CommandStatus": {
                    "StringValue": "Success" if success else "Error",
                    "DataType": "String",
                },
                "InstanceId": {"StringValue": self.instance_id, "DataType": "String"},
                "CommandMessageId": {"StringValue": message_id, "DataType": "String"},
            },
        )

    @staticmethod
    def socat(port):
        cmd = (
            '/usr/bin/socat -vv TCP-LISTEN:%d,crlf,reuseaddr,fork SYSTEM:"echo HTTP/1.0 200; echo Content-Type\: text/plain; echo; echo OK %d"'
            % (port, port)
        )
        logger.info(cmd)

        proc = subprocess.Popen(cmd, shell=True)

        if proc.returncode:
            cout, cerr = proc.communicate()
            raise subprocess.CalledProcessError(
                returncode=proc.returncode, cmd=cmd, output=cout, stderr=cerr
            )

        return proc.pid

    @staticmethod
    def ping_check(address):
        cmd = "ping -c 5 %s" % address
        logger.info(cmd)

        subprocess.run(cmd.split(), check=True)

        return True

    @staticmethod
    def tcp_check(address):
        url = "http://%s" % address
        logger.info("Connecting to %s" % url)

        res = urllib.request.urlopen(url).readline().decode().strip()

        if res == "OK %s" % address.split(":")[1]:
            logger.info("Result OK")

            return True
        return False

    @staticmethod
    def url_check(url):
        logger.info("Connecting to %s" % url)

        res = urllib.request.urlopen(url)

        if res.status == 200:
            logger.info("Result OK")

            return True
        return False

    def handle(self, message):
        commands = [
            (cmd, args["StringValue"])
            for cmd, args in message.message_attributes.items()
            if args["DataType"] == "String.Command"
        ]

        for cmd in commands:
            try:
                if cmd[0] == "Listen":
                    logger.info("Listen on %d" % int(cmd[1]))
                    if cmd[1] not in self._listen:
                        pid = self.socat(int(cmd[1]))
                        self._listen[cmd[1]] = pid

                    self.response(
                        success=True, body=message.body, message_id=message.message_id
                    )

                if cmd[0] == "ConnectTCP":
                    res = self.tcp_check(cmd[1])

                    if res:
                        self.response(
                            success=True,
                            body=message.body,
                            message_id=message.message_id,
                        )

                if cmd[0] == "Ping":
                    res = self.ping_check(cmd[1])

                    if res:
                        self.response(
                            success=True,
                            body=message.body,
                            message_id=message.message_id,
                        )

                if cmd[0] == "ConnectURL":
                    res = self.url_check(cmd[1])

                    if res:
                        self.response(
                            success=True,
                            body=message.body,
                            message_id=message.message_id,
                        )
            except Exception as e:
                self.response(success=False, body=str(e), message_id=message.message_id)

    def loop(self):
        while True:
            logger.info("Polling %s" % self.command_queue.url)
            response = self.command_queue.receive_messages(
                AttributeNames=["All"],
                MaxNumberOfMessages=1,
                MessageAttributeNames=["All"],
                WaitTimeSeconds=20,
            )
            try:
                try:
                    self.handle(message=response[0])
                    response[0].delete()
                except AttributeError:
                    logger.info("Deleting ", response[0].receipt_handle)
                    response[0].delete()
            except (KeyError, IndexError):
                print(response)



import boto3
import botocore
import logging
import time
import json
import uuid
from contextlib import contextmanager

LOG = logging.getLogger(__name__)


class ConnectivityTestOrchestrator(object):
    def __init__(self, queue_name="connectivitytest", sqs_region="us-east-2"):
        self.queue_name = "%s-%s" % (queue_name, uuid.uuid1())
        self._timeout = 20
        self._registered_instances = {}
        self._response_tracker = {}
        self._name_to_instance_map = {}
        self._instance_to_name_map = {}
        self.sqs = boto3.resource("sqs", region_name=sqs_region)

        try:
            # Get the queue. This returns an SQS.Queue instance
            self.registration_queue = self.sqs.get_queue_by_name(
                QueueName=self.queue_name
            )

            assert self.registration_queue.url
        except botocore.exceptions.ClientError as err:
            self.registration_queue = self.sqs.create_queue(QueueName=self.queue_name)

    def cleanup(self):
        self.registration_queue.delete()
        for instance_id, instance_info in self._registered_instances.items():
            instance_info["command_queue"].delete()

    def instance_str(self, instance_id):
        return "%s (%s)" % (instance_id, self._instance_to_name_map[instance_id])

    def receive(self, max_number=1):
        messages = self.registration_queue.receive_messages(
            AttributeNames=["All"],
            MaxNumberOfMessages=max_number,
            MessageAttributeNames=["All"],
            WaitTimeSeconds=self._timeout,
        )

        return messages

    def send_command(self, instance, command, value):
        instance_id = self._name_to_instance_map[instance]

        res = self._registered_instances[instance_id]["command_queue"].send_message(
            MessageBody="Connection Test: %s" % value,
            MessageAttributes={
                command: {"StringValue": value, "DataType": "String.Command"}
            },
        )

        self._response_tracker[res["MessageId"]] = False
        LOG.info("Sent %s %s to %s" % (command, value, self.instance_str(instance_id)))

    def wait_for_registrations(self, instance_ids=None, instances=None, max_retry=20):
        """
        Try to register ec2 instances for connection testing. Those instances must run the python tooling and
        initiate the registration process, we just wait for all of them. Returns false if not all of them registered.

        :param instance_ids: List of instance ids that we want to use for testing OR
        :type list
        :param instances:  Dict of name: instance id pairs that we want to use for testing
        :type dict
        :param max_retry: Max number of retries till the method polls SQS
        :type int
        :return: bool
        """
        if not instance_ids and instances:
            instance_ids = [instance[1] for instance in instances.items()]
            self._name_to_instance_map = instances
        else:
            self._name_to_instance_map = {
                instance_id: instance_id for instance_id in instance_ids
            }
        self._instance_to_name_map = {
            value: key for key, value in self._name_to_instance_map.items()
        }
        self._registered_instances = {
            instance_id: False for instance_id in instance_ids
        }

        LOG.info("Waiting for instance registrations: %s" % instance_ids)
        i = 0
        while i < max_retry:
            i = i + 1

            LOG.info("Receiving registration messages from SQS")
            messages = self.receive(10)
            if messages:
                for message in messages:
                    if "InstanceRegistration" not in message.message_attributes:
                        continue

                    instance_id = message.message_attributes["InstanceRegistration"][
                        "StringValue"
                    ]
                    if instance_id in instance_ids:
                        self._registered_instances[instance_id] = json.loads(
                            message.body
                        )

                        self._registered_instances[instance_id][
                            "command_queue"
                        ] = self.sqs.get_queue_by_name(
                            QueueName=self._registered_instances[instance_id][
                                "command_queue_name"
                            ]
                        )

                        LOG.info(
                            "Instance %s (%s) successfully registered"
                            % (self._instance_to_name_map[instance_id], instance_id)
                        )

                        message.delete()

                    if all([el[1] for el in self._registered_instances.items()]):
                        return True

            if i >= 5:
                time.sleep(10)

        return False

    def wait_for_response(self, max_retry=20):
        LOG.info("Waiting for instances to respond")

        i = 0
        while i < max_retry:
            i = i + 1

            messages = self.receive(10)
            if messages:
                for message in messages:
                    if "CommandStatus" not in message.message_attributes:
                        continue

                    instance_id = message.message_attributes["InstanceId"][
                        "StringValue"
                    ]
                    command_message_id = message.message_attributes["CommandMessageId"][
                        "StringValue"
                    ]
                    status = message.message_attributes["CommandStatus"]["StringValue"]
                    body = message.body

                    if command_message_id in self._response_tracker:
                        self._response_tracker[command_message_id] = True
                        message.delete()
                    else:
                        continue

                    if status != "Success":
                        LOG.error(
                            "Got message response %s (%s) from %s"
                            % (status, body, self.instance_str(instance_id))
                        )
                        return False

                    LOG.info(
                        "Got message response %s from %s"
                        % (status, self.instance_str(instance_id))
                    )
                    if all([el[1] for el in self._response_tracker.items()]):
                        return True

            if i >= 5:
                time.sleep(10)

        return False

    def get_instance_ips(self, instance, public=False):
        instance_id = self._name_to_instance_map[instance]

        nic_config = self._registered_instances[instance_id]["nic_config"]
        if public:
            ip_list = nic_config["ipv4-associations"]
        else:
            ip_list = nic_config["local-ipv4s"]

        return ip_list

    def test_url(self, instance, url, batch=False):
        self.send_command(instance=instance, command="ConnectURL", value=url)

        if batch:
            return True
        return self.wait_for_response()

    def test_ping_internal(self, source, destination, batch=False):
        for destination_ip in self.get_instance_ips(destination):
            self.send_command(instance=source, command="Ping", value=destination_ip)

        if batch:
            return True
        return self.wait_for_response()

    def test_tcp(self, source, destination, port, public=False):
        self.send_command(instance=destination, command="Listen", value=str(port))
        self.wait_for_response()

        for destination_ip in self.get_instance_ips(destination, public=public):
            address = "%s:%d" % (destination_ip, port)
            self.send_command(instance=source, command="ConnectTCP", value=address)

        return self.wait_for_response()


@contextmanager
def connectivity_tester():
    connectivity_test_object = ConnectivityTestOrchestrator()

    try:
        yield connectivity_test_object
    finally:
        connectivity_test_object.cleanup()



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=7.0', 'python-daemon>=2.2.4', 'boto3>=1.10' ]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest>=3', ]

setup(
    author="revenants-cie",
    author_email='dev@revenants.net',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Test network connections between multiple instances and the internet using standard unit tests",
    entry_points={
        'console_scripts': [
            'network-connectivity-tester-aws-client=network_connectivity_tester.cli.awsclient:main',
        ],
    },
    install_requires=requirements,
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='network_connectivity_tester',
    name='network_connectivity_tester',
    packages=find_packages(include=['network_connectivity_tester', 'network_connectivity_tester.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/revenants-cie/network_connectivity_tester',
    version='0.1.1',
    zip_safe=False,
)
